//      tsortbui.cpp - test program for tsortbui.h

#include "tsortbui.h"
#include <fstream>
#include <iostream>
#include <stdlib.h>
#include <time.h>

typedef unsigned long long uint64_t;

#define QP 1        // if != 0, use queryperformance for timer

#define COUNT (16*1024*1024)    // number of values to sort

#if QP
#include <math.h>
#include <windows.h>
#pragma comment(lib, "winmm.lib")
typedef LARGE_INTEGER LI64;
#endif

#if QP
static LI64     liQPFrequency;  // cpu counter values
static LI64     liStartTime;
static LI64     liStopTime;
static double   dQPFrequency;
static double   dStartTime;
static double   dStopTime;
static double   dElapsedTime;
#else
clock_t ctTimeStart;            // clock values
clock_t ctTimeStop;
#endif

int main(int argc, char**argv)
{
    uint64_t   r;               // random number
    size_t     i;

    uint64_t * a = new uint64_t [COUNT];    // allocate data array

    for(i = 0; i < COUNT; i++){
        r  = (((uint64_t)((rand()>>4) & 0xff))<< 0);
        r += (((uint64_t)((rand()>>4) & 0xff))<< 8);
        r += (((uint64_t)((rand()>>4) & 0xff))<<16);
        r += (((uint64_t)((rand()>>4) & 0xff))<<24);
        r += (((uint64_t)((rand()>>4) & 0xff))<<32);
        r += (((uint64_t)((rand()>>4) & 0xff))<<40);
        r += (((uint64_t)((rand()>>4) & 0xff))<<48);
        r += (((uint64_t)((rand()>>4) & 0xff))<<56);
        a[i] = r;
    }

#if QP
    QueryPerformanceFrequency(&liQPFrequency);
    dQPFrequency = (double)liQPFrequency.QuadPart;
    timeBeginPeriod(1);                     // set ticker to 1000 hz
    Sleep(128);                             // wait for it to settle
    QueryPerformanceCounter(&liStartTime);
#else
    ctTimeStart = clock();
#endif

    BottomUpMergeSort(a, COUNT);

#if QP
    QueryPerformanceCounter(&liStopTime);
    dStartTime = (double)liStartTime.QuadPart;
    dStopTime  = (double)liStopTime.QuadPart;
    dElapsedTime = (dStopTime - dStartTime) / dQPFrequency;
    timeEndPeriod(1);                       // restore ticker to default
    std::cout << "# of seconds " << dElapsedTime << std::endl;
#else
    ctTimeStop = clock();
    std::cout << "# of ticks " << ctTimeStop - ctTimeStart << std::endl;
#endif

    for(i = 1; i < COUNT; i++){
        if(a[i-1] > a[i]){
            std::cout << "error" << std::endl;
            break;
        }
    }

    delete[] a;

    return(0);
}