//      tsorttd.h - top down merge sort

template <class T>
void TopDownMergeSortAtoA(T a[], T b[], size_t ll, size_t ee);
template <class T>
void TopDownMergeSortAtoB(T a[], T b[], size_t ll, size_t ee);
template <class T>
void TopDownMerge(T a[], T b[], size_t ll, size_t rr, size_t ee);

// insertion sort size
#define ISZ_ 64

template <class T>
void TopDownMergeSort(T a[], size_t n)      // entry function
{
    if(n < 2)                               // if size < 2 return
        return;
    T *b = new T[n];
    TopDownMergeSortAtoA(a, b, 0, n);
    delete[] b;
}

template <class T>
void TopDownMergeSortAtoA(T a[], T b[], size_t ll, size_t ee)
{
    if (ee - ll <= ISZ_){                   // check for insertion sort
        size_t l, r;
        size_t i, j;
        T t;
        for (l = ll; l < ee; l = r) {
            r = l + ISZ_;
            if (r > ee)r = ee;
            l--;
            for (j = l + 2; j < r; j++) {
                t = a[j];
                i = j-1;
                while(i != l && a[i] > t){
                    a[i+1] = a[i];
                    i--;
                }
                a[i+1] = t;
            }
        }
        return;
    }

    size_t rr = (ll + ee)>>1;               // midpoint, start of right half
    TopDownMergeSortAtoB(a, b, ll, rr);
    TopDownMergeSortAtoB(a, b, rr, ee);
    TopDownMerge(b, a, ll, rr, ee);         // merge b to a
}

template <class T>
void TopDownMergeSortAtoB(T a[], T b[], size_t ll, size_t ee)
{
    if (ee - ll <= ISZ_){                   // check for insertion sort
        size_t l, r;
        size_t i, j;
        T t;
        for (l = ll; l < ee; l = r) {
            r = l + ISZ_;
            if (r > ee)r = ee;
            l--;
            for (j = l + 2; j < r; j++) {
                t = a[j];
                i = j-1;
                while(i != l && a[i] > t){
                    a[i+1] = a[i];
                    i--;
                }
                a[i+1] = t;
            }
        }
        for(i = ll; i < ee; i++)            // copy a to b
            b[i] = a[i];
        return;
    }

    size_t rr = (ll + ee)>>1;               // midpoint, start of right half
    TopDownMergeSortAtoA(a, b, ll, rr);
    TopDownMergeSortAtoA(a, b, rr, ee);
    TopDownMerge(a, b, ll, rr, ee);         // merge a to b
}

template <class T>
void TopDownMerge(T a[], T b[], size_t ll, size_t rr, size_t ee)
{
    size_t o = ll;                          // b[]       index
    size_t l = ll;                          // a[] left  index
    size_t r = rr;                          // a[] right index

    while(1){                               // merge data
        if(a[l] <= a[r]){                   // if a[l] <= a[r]
            b[o++] = a[l++];                //   copy a[l]
            if(l < rr)                      //   if not end of left run
                continue;                   //     continue (back to while)
            while(r < ee){                  //   else copy rest of right run
                b[o++] = a[r++];
            }
            break;                         //     and return
        } else {                            // else a[l] > a[r]
            b[o++] = a[r++];                //   copy a[r]
            if(r < ee)                      //   if not end of right run
                continue;                   //     continue (back to while)
            while(l < rr){                  //   else copy rest of left run
                b[o++] = a[l++];
            }
            break;                         //     and return
        }
    }
}
